export declare class Generator {
    /**
     * The generators epoch timestamp in milliseconds.
     *
     * Defaults to "1st of January, 2000, 00:00".
     *
     * @type {number}
     */
    EPOCH: number;
    /**
     * The id of the shard running this generator.
     *
     * Defaults to "1".
     *
     * @type {number}
     */
    SHARD_ID: number;
    /**
     * The current increment iteration this generator is on.
     * @type {number}
     */
    INCREMENT: number;
    /**
     * The main class used for generating snowflakes.
     * @param {Date|number} [epoch] - The epoch timestamp (in milliseconds) to generate from
     * @param {number} [shardID] - Useful when running multiple generators at the same time to prevent duplicates.
     */
    constructor(epoch?: Date | number, shardID?: number);
    /**
     * Generates a single snowflake.
     * @param {number} timestamp - Timestamp to generate from
     * @returns {bigint}
     * @private
     */
    static _generate(generator: Generator, timestamp: number): bigint;
    /**
     * Generates snowflakes.
     * @param {number} [amount = 1] - Amount of snowflakes to generate, recommended not to go above `1024` or duplicates will arise.
     * @param {Date|number} [timestamp = Date.now] - Timestamp to generate from
     * @returns {bigint|bigint[]}
     */
    generate(amount?: number, timestamp?: Date | number): bigint | bigint[];
    /**
     * Deconstruct a snowflake to its values.
     * @param {SnowflakeResolvable} snowflake - Snowflake to deconstruct
     * @returns {DeconstructedSnowflake}
     * @private
     */
    static _deconstruct(generator: Generator, snowflake: SnowflakeResolvable): DeconstructedSnowflake;
    /**
     * Deconstruct a snowflake to its values using the `Generator.epoch`.
     * @param {SnowflakeResolvable|SnowflakeResolvable[]} snowflake - Snowflake(s) to deconstruct
     * @returns {DeconstructedSnowflake|DeconstructedSnowflake[]}
     */
    deconstruct(snowflake: SnowflakeResolvable | SnowflakeResolvable[]): DeconstructedSnowflake | DeconstructedSnowflake[];
    /**
     * Extract bits and their values from a snowflake.
     * @param {SnowflakeResolvable} snowflake - Snowflake to extract from
     * @param {number|bigint} shift - Number of bits to shift before extracting
     * @param {number|bigint} length - Number of bits to extract before stopping
     * @returns {bigint}
     */
    static extractBits(snowflake: SnowflakeResolvable, shift: number | bigint, length: number | bigint): bigint;
    /**
     * Transform a snowflake into its 64Bit binary string.
     * @param {SnowflakeResolvable} snowflake - Snowflake to transform
     * @returns {string}
     * @private
     */
    static binary(snowflake: SnowflakeResolvable): string;
}
/**
 * Resolvable value types for a valid Snowflake.
 * * string
 * * number
 * * bigint
 * @type {SnowflakeResolvable}
 */
declare type SnowflakeResolvable = string | number | bigint;
/**
 * Interface of a Snowflake after `Generator.deconstruct()`.
 * @property {bigint} snowflake - Snowflake deconstructed from
 * @property {bigint} timestamp - The timestamp the snowflake was generated
 * @property {bigint} shard_id - The shard_id used when generating
 * @property {bigint} increment - The increment of this snowflake
 * @property {string} binary - The 64Bit snowflake binary string
 * @interface DeconstructedSnowflake
 */
interface DeconstructedSnowflake {
    snowflake: bigint;
    timestamp: bigint;
    shard_id: bigint;
    increment: bigint;
    binary: string;
}
export {};
